//const
var STATE_INIT = "i";
var STATE_TITLE = "t";
var STATE_BUILD = "b";
var STATE_SHOOT = "s";
var STATE_COMPLETED = "c";
var STATE_CREATIVE = "cr";

var p;

function Director() {
	//state
	this.state = "i";
	this.prevState = "i";
	this.tick = 0;
	this.totalLevels = 40;
	this.currentLevel = 1;
	this.selectedLevel = 1;
	this.currentDraggedPart = null;
	this.currentMode = "assemble";
	this.currentTargetsDestroyed = 0;
	this.bulletsFired = 0;
	this.adModulo = 0;
	
	this.partList = [];
	this.targetList = [];
	
	this.creativeBtn = new PIXI.Sprite.fromFrame("button/creative.png");
	
	var self = this;
	
	this.levelTxt = new PIXI.BitmapText("LEVEL 1", {font: "30px Coolvetica30" });
	this.levelTxt.tint = 0xC7F464;
	this.levelTxt.anchor.set(0.5);
	this.levelTxt.x = screenManager.centerX;
	this.levelTxt.y = 40;
	
	this.buildImg = new PIXI.Sprite.fromFrame("build.png");
	this.buildImg.anchor.set(0.5);
	this.buildImg.alpha = 0;
	this.buildImg.x = screenManager.centerX;
	this.buildImg.y = screenManager.centerY;
	
	this.shootImg = new PIXI.Sprite.fromFrame("shoot.png");
	this.shootImg.anchor.set(0.5);
	this.shootImg.alpha = 0;
	this.shootImg.x = screenManager.centerX;
	this.shootImg.y = screenManager.centerY;
	
	this.creativeBtn.buttonMode = true;
	this.creativeBtn.interactive = true;
	this.creativeBtn.on('pointerdown', function(e) { self.goCreative(); });
	
	this.shootingBtn = new PIXI.Sprite.fromFrame("button/shooting.png");
	
	this.shootingBtn.buttonMode = true;
	this.shootingBtn.interactive = true;
	this.shootingBtn.on('pointerdown', function(e) { self.goShoot(); });
	
	this.assembleBtn = new PIXI.Sprite.fromFrame("button/assemble.png");
	
	this.assembleBtn.buttonMode = true;
	this.assembleBtn.interactive = true;
	this.assembleBtn.on('pointerdown', function(e) { self.goBuild(); });
	
	this.prevBtn = new PIXI.Sprite.fromFrame("prev.png");
	this.prevBtn.buttonMode = true;
	this.prevBtn.interactive = true;
	this.prevBtn.on('pointerdown',function(e) { self.goPrev(); });
	
	this.nextBtn = new PIXI.Sprite.fromFrame("next.png");
	this.nextBtn.buttonMode = true;
	this.nextBtn.interactive = true;
	this.nextBtn.on('pointerdown', function(e) { self.goNext(); });
	
	layers.ui.addChild(this.creativeBtn);
	layers.ui.addChild(this.shootingBtn);
	layers.ui.addChild(this.assembleBtn);
	layers.ui.addChild(this.shootImg);
	layers.ui.addChild(this.buildImg);
	layers.ui.addChild(this.levelTxt);
	layers.ui.addChild(this.prevBtn);
	layers.ui.addChild(this.nextBtn);
	
	//CHEATSHEET
	//this.emitter = new Emitter(["smoke.png"], unitLayer, 0, Math.PI, 3, -5, 4, true); //(imgs, layer, rate, angle, amp, v, life, explode)
	//this.bodySprite = new PIXI.Sprite.fromFrame("pjBody.png");
	
	this.resize();
}

Director.prototype.start = function() {
	$(".loadBar").remove(); //Hide loading bar 
	$("img").remove(); //Hide sponsor branding
	
	this.goTitle();
}

Director.prototype.resize = function() {
	this.levelTxt.x = screenManager.centerX;
	this.levelTxt.y = 40;
	
	this.prevBtn.x = this.levelTxt.x - 89;
	this.nextBtn.x = this.levelTxt.x + 67;
	
	this.prevBtn.y = this.nextBtn.y = 24;
	
	this.buildImg.x = this.shootImg.x = screenManager.centerX;
	this.buildImg.y = this.shootImg.y = 140;
	
	this.creativeBtn.x = screenManager.windowW - this.creativeBtn.width;
	this.creativeBtn.y = screenManager.centerY - this.creativeBtn.height * 0.5;
	
	this.shootingBtn.x = screenManager.windowW - this.shootingBtn.width;
	this.shootingBtn.y = screenManager.centerY - this.shootingBtn.height;
	
	this.assembleBtn.x = screenManager.windowW - this.assembleBtn.width;
	this.assembleBtn.y = screenManager.centerY;
}

Director.prototype.goTitle = function() {
	this.setState(STATE_TITLE);
	sound.play(STATE_TITLE);
}

Director.prototype.goBuild = function() {
	if(this.state != STATE_INIT) {
		sound.play(STATE_BUILD);
		this.setState(STATE_BUILD);
		
		if(this.prevState == STATE_TITLE && tutorial.step == 1 && this.currentLevel == 1) {
			tutorial.show();
		}
		
		this.tick = 0;
		this.currentMode = "assemble";
		this.removeTargets();
		menu.setHeight();
		gun.clear();
		level.setData(this.selectedLevel.toString());
	}else{
		this.start();
	}
}

Director.prototype.goCreative = function() {
	if(this.state == STATE_BUILD) gun.clear();
	this.setState(STATE_CREATIVE);
	this.currentMode = "creative";
	this.tick = 0;
	menu.setHeight();
	level.removeParts();
	this.removeTargets();
	gun.holder.pivot.x = gun.holder.pivot.y = gun.holder.x = gun.holder.y = gun.x = gun.y = 0;
}

Director.prototype.goShoot = function() {
	this.adModulo++;
	if(this.adModulo % 4 == 0 && (isLagged || isMobile)) {
		this.showAd();
	}
	this.setState(STATE_SHOOT);
	this.tick = -100;
	this.currentTargetsDestroyed = 0;
	
	gun.setStats();
	save.save();
	observer.event(EVENT_SHOOT_MODE);
}

Director.prototype.showAd = function() {
	if(isLagged) {
		if(typeof prerollStart === 'undefined'){
			console.log('skip ad, prerollStart not found');
		}else{
			LaggedAPI.APIAds.show('interstitial','gun-builder-2','gun-builder-2-game.jpg',function(response) {
				if(response.success) {
				console.log('ad done');
				}else {
				console.log('ad error, continue');
				}
			});
		}
	}else{
		showInterstitial();
	}
}

Director.prototype.levelCompleted = function() {
	this.selectedLevel++;
	if(this.selectedLevel == 6) {
		achievements.earn("guns5");
	}
	if(this.selectedLevel == 11) {
		achievements.earn("guns10");
	}
	if(this.selectedLevel == 16) {
		achievements.earn("guns15");
	}
	if(this.selectedLevel == 21) {
		achievements.earn("guns20");
	}
	if(this.selectedLevel > this.totalLevels) this.selectedLevel = this.totalLevels;
	if(this.selectedLevel > this.currentLevel) this.currentLevel = this.selectedLevel;
	
	this.levelTxt.text = "LEVEL " + this.selectedLevel;
	completed.show();
	this.tick = 0;
	this.setState(STATE_COMPLETED);
	save.save();
}

Director.prototype.goNext = function() {
	level.removeParts();
	this.selectedLevel++;
	if(this.selectedLevel > this.totalLevels) this.selectedLevel = this.totalLevels;
	this.levelTxt.text = "LEVEL " + this.selectedLevel;
	this.goBuild();
}

Director.prototype.goPrev = function() {
	level.removeParts();
	this.selectedLevel--;
	if(this.selectedLevel < 1) this.selectedLevel = 1;
	this.levelTxt.text = "LEVEL " + this.selectedLevel;
	this.goBuild();
}

Director.prototype.nextLevel = function() {
	var i, p;
	for(i =  this.partList.length - 1 ; i >= 0; i--) {
		p = this.partList[i];
		p.remove();
	}
	
	this.goBuild();
}

Director.prototype.findPartType = function(type) {
	var i, p;
	var list = [];
	for(i =  this.partList.length - 1 ; i >= 0; i--) {
		p = this.partList[i];
		if(p.data.type == type) list.push(p);
	}
	return list;
}

Director.prototype.findExcessParts = function() {
	var i, p;
	var list = [];
	for(i =  this.partList.length - 1 ; i >= 0; i--) {
		p = this.partList[i];
		if(!p.completed) list.push(p);
	}
	return list;
}

Director.prototype.findBulletCaseGenerators = function() {
	var i, p;
	var list = [];
	for(i =  this.partList.length - 1 ; i >= 0; i--) {
		p = this.partList[i];
		if(p.data.bulletCase != null) list.push(p);
	}
	return list;
}

Director.prototype.findUnlockedPartsData = function() {
	return embed.parts.findAll(["unlock"], [this.currentLevel]);
}

Director.prototype.createTargets = function() {
	var i = 0; 
	var px = [155, 70, 155];
	var py = [125, 305, 485];
	for(i = 0 ; i < 3; i++) {
		this.targetList.push(new Target(px[i], py[i]));
	}
}

Director.prototype.removeTargets = function() {
	var i, t;
	for(i = this.targetList.length - 1; i >= 0; i--) {
		t = this.targetList[i];
		t.remove();
	}
}


Director.prototype.setState = function(_state) {
	this.prevState = this.state;
	this.state = _state;
}

Director.prototype.update = function() {
	this.tick++;
	
	var i, p, t;
	for(i =  this.partList.length - 1 ; i >= 0; i--) {
		p = this.partList[i];
		p.update();
	}
	
	for(i = this.targetList.length - 1; i >= 0; i--) {
		t = this.targetList[i];
		t.update();
	}
	
	switch(this.state) {
		case STATE_INIT:
		case STATE_TITLE:
			this.levelTxt.y += (-10 - this.levelTxt.y) * 0.2;
			this.levelTxt.visible = false;
			this.buildImg.visible = false;
			this.creativeBtn.visible = false;
			this.shootingBtn.visible = false;
			this.assembleBtn.visible = false;
			this.shootImg.visible = false;
			this.prevBtn.visible = false;
			this.nextBtn.visible = false;
			break;
			
		case STATE_SHOOT:
			if(this.targetList.length == 0 && this.tick > 20) {
				this.createTargets();
			}
			if(this.tick < 0) {
				this.shootImg.alpha += (1 - this.shootImg.alpha) * 0.3;
			}else{
				this.shootImg.alpha += (0 - this.shootImg.alpha) * 0.3;
			}
			this.levelTxt.visible = false;
			this.shootImg.visible = true;
			this.buildImg.visible = false;
			this.creativeBtn.visible = false;
			this.shootingBtn.visible = false;
			this.assembleBtn.visible = false;
			this.prevBtn.visible = false;
			this.nextBtn.visible = false;
			break;
			
		case STATE_BUILD:
			if(this.tick > 40 && this.tick < 120) {
				this.buildImg.alpha += (1 - this.buildImg.alpha) * 0.3;
			}else{
				this.buildImg.alpha += (0 - this.buildImg.alpha) * 0.3;
			}
			this.levelTxt.y += (40 - this.levelTxt.y) * 0.2;
			this.levelTxt.visible = true;
			this.buildImg.visible = true;
			this.shootImg.visible = false;
			if(this.currentLevel > 1 && this.tick > 140) {
				this.creativeBtn.visible = true;
				this.creativeBtn.x += (screenManager.windowW - this.creativeBtn.width - this.creativeBtn.x) * 0.2;
			}else{
				this.creativeBtn.x = screenManager.windowW;
			}
			this.shootingBtn.visible = false;
			this.assembleBtn.visible = false;
			this.prevBtn.visible = (this.selectedLevel > 1);
			this.nextBtn.visible = (this.selectedLevel < this.currentLevel);
			break;
			
		case STATE_COMPLETED:
			this.levelTxt.y += (-10 - this.levelTxt.y) * 0.2;
			this.levelTxt.visible = false;
			this.buildImg.visible = false;
			this.shootImg.visible = false;
			this.creativeBtn.visible = false;
			this.shootingBtn.visible = false;
			this.assembleBtn.visible = false;
			this.prevBtn.visible = false;
			this.nextBtn.visible = false;
			break;
			
		case STATE_CREATIVE:
			this.levelTxt.y += (-10 - this.levelTxt.y) * 0.2;
			this.levelTxt.visible = false;
			this.buildImg.visible = false;
			this.shootImg.visible = false;
			this.creativeBtn.visible = false;
			if(this.tick > 100) {
				if(this.partList.length > 0) {
					this.shootingBtn.x += (screenManager.windowW - this.shootingBtn.width - this.shootingBtn.x) * 0.2;
				}else{
					this.shootingBtn.x  += (screenManager.windowW - this.shootingBtn.x) * 0.2;
				}
				if(this.tick > 120) this.assembleBtn.x += (screenManager.windowW - this.assembleBtn.width - this.assembleBtn.x) * 0.2;
			}else{
				this.shootingBtn.x = screenManager.windowW;
			}
			this.shootingBtn.visible = true;
			this.assembleBtn.visible = true;
			this.prevBtn.visible = false;
			this.nextBtn.visible = false;
			break;
	}
	
}
