//EMITTER
function Emitter(imgs, layer, rate, angle, amp, v, life, explode) {
	this.tick = 0;
	this.imgs = imgs;
	this.rate = rate;
	this.angle = angle;
	this.amp = amp;
	this.v = v;
	this.life = life;
	this.explode = explode;
	this.layer = layer;
	this.timer = 0;
	this.x = this.y = 0;
	this.enabled = true;
	this.particles = [];
	this.pool = [];
}

Emitter.prototype.emit = function() {
	this.particles.push(this.getParticle());
};

Emitter.prototype.getParticle = function(d) {
	//OBJECT POOLING
	if(this.pool.length > 0){
		var p = this.pool[0];
		p.init();
		this.pool.splice(0, 1);
		return p;
	}
	
	return new Particle(this);
};

Emitter.prototype.updateParticles = function() {
	var i;
	for(i = this.particles.length - 1; i >= 0; i--) {
		var p = this.particles[i];
		p.update();
		p.updateRemove();
	}
};

Emitter.prototype.update = function() {
	this.tick++;
	
	if(this.rate == 0) {
		while(this.timer > 0) {
			this.tick++;
			if(this.tick > this.rate && this.enabled && this.timer > 0) {
				this.emit();
				this.tick = 0;
			}
			this.timer--;
		}
	}
	if(this.tick > this.rate && this.enabled && this.explode == null) {
		this.emit();
		this.tick = 0;
	}
	this.updateParticles();
	this.timer--;
};

//// PARTICLE
function Particle(emitter) {
	var img = emitter.imgs[Math.floor(Math.random() * emitter.imgs.length)];
	this.sprite = new PIXI.Sprite.fromFrame(img);
	if(img == "smoke.png") {
		this.sprite.blendMode = PIXI.BLEND_MODES.OVERLAY;
		this.sprite.scale.x = this.sprite.scale.y = Math.random() * 0.5 + 0.5;
	}
	this.sprite.anchor.set(0.5);
	emitter.layer.addChild(this.sprite);
	this.emitter = emitter;
	
	this.init();
}

Particle.prototype.init = function() {
	this.sprite.x = this.emitter.x;
	this.sprite.y = this.emitter.y;
	
	var a = this.emitter.angle + Math.random() * this.emitter.amp - this.emitter.amp * 0.5;
	this.angle = a;
	this.vx = Math.sin(a) * (this.emitter.v * (0.25 + Math.random() * 0.75));
	this.vy = Math.cos(a) * (this.emitter.v * (0.25 + Math.random() * 0.75));
	this.vr = Math.random() * 0.3 - 0.15;
	if(this.emitter.imgs[0] == "bulletCollision.png") {
		this.vr = 0;
		this.sprite.rotation = -a;
	}
	
	this.maxLife = this.life = this.emitter.life + Math.random() * this.emitter.life * 1.5;
	this.remove = false;
};

Particle.prototype.updateRemove = function() {
	if(this.remove) {
		this.sprite.y = screenManager.windowH + this.sprite.height + 100;
		this.sprite.alpha = 0;
		this.emitter.pool.push(this);
		this.emitter.particles.splice(this.emitter.particles.indexOf(this), 1);
	}
};

Particle.prototype.update = function() {
	this.life--;
	
	if(Math.floor(this.life) == 0) {
		this.remove = true;
	}
	var percent = this.life / (20);
	if(percent > 1) percent = 1;
	this.sprite.alpha = percent;
	this.sprite.x += this.vx;
	this.sprite.y += this.vy;
	this.sprite.rotation += this.vr;
	
	this.vx = this.vx * 0.96;// - this.maxLife * 0.03;
	this.vy = this.vy * 0.96;// - this.maxLife * 0.03;
};