/**
 * ...
 * @author lafunk
 */


//STATIC VARS
//SYSTEM
var savedata;
var settings;
var sound;
var apis;
var embed;
var observer;
var tracker;

//CORE
var screenManager;
var layers;
var input;
var save;
var achievements;

//GAME
var title;
var bg;
var level;
var tutorial;
var player;
var menu;
var completed;
var gun;
var exportBtn;
var importBtn;

var director;

function Game() {
	this.init();
}

Game.prototype.init = function() {
	//CORE
	embed = new Embed();
	input = new Input();
	sound = new Sound();
	screenManager = new ScreenManager();
	save = new SaveData();
	achievements = new Achievements();
	
	//GAME
	layers = new Layers();
	level = new Level();
	tutorial = new Tutorial();
	title = new Title();
	bg = new Bg();
	menu = new Menu();
	completed = new Completed();
	gun = new Gun();
	exportBtn = new Export();
	importBtn = new Import();
	settings = new Settings();
	
	observer = new Observer();
	
	director  = new Director();
	
	observer.addListener(tutorial);
	
	window.requestAnimationFrame(loop);
	var self = this;
	window.onresize = function(event) { self.resize(); };
	
	this.resize();
	
	save.load();
}

Game.prototype.resize = function() {
	screenManager.resize();
	
	//if(screenManager.windowW < 800) {
		main.app.renderer.resize(screenManager.windowW, screenManager.windowH);
		main.app.renderer.view.style.width =  screenManager.screenW + 'px';
		main.app.renderer.view.style.height = screenManager.screenH + 'px';
	/*}else{
		main.app.renderer.resize(800, 600);
		main.app.renderer.view.style.width =  800 + 'px';
		main.app.renderer.view.style.height = 600 + 'px';
	}*/
	director.resize();
	menu.resize();
	gun.resize();
	completed.resize();
	title.resize();
	bg.resize();
	settings.resize();
	exportBtn.resize();
	importBtn.resize();
}

function loop() {
	if(Date.now() - lastFrame > frameLength) {
		lastFrame = Date.now();
		update();
	}
	requestAnimationFrame(loop);
}

function update() {
	if(debug) stats.begin();
	
	title.update();
	director.update();
	level.update();
	menu.update();
	completed.update();
	gun.update();
	tutorial.update();
	exportBtn.update();
	importBtn.update();
	settings.update();
	
	if(debug) stats.end();
}
	