function Gun() {
	this.dragged = false;
	this.holder = new PIXI.DisplayObjectContainer();
	this.dragStart = new PIXI.Point(0, 0);
	this.bulletList = [];
	this.bulletCaseList = [];
	this.muzzleList = [];
	this.smokeEmitterList = [];
	this.collisionEmitter = new Emitter(["bulletCollision.png"], layers.main, 0, Math.PI * 0.5, 0.75, 23, 12, true);
	
	this.magSize = 6;
	this.magBullets = 6;
	this.accuracy = 1;
	this.fireMode = "semi";
	this.fireRate = 0;
	this.cooldown = 0;
	this.reloadTime = 0;
	this.damage = 1;
	this.currentBurst = 0;
	
	this.x = this.y = 0;
	
	this.flashFilter = new PIXI.filters.ColorMatrixFilter();
	
	this.holder.buttonMode = true;
	this.holder.interactive = true;
	
	var self = this;
	this.holder.on('pointerdown', function(e) { self.onDragStart(e); })
    this.holder.on('pointerup',  function() { self.onDragStop(); });
    this.holder.on('pointerupoutside',  function() { self.onDragStop(); });
	
	layers.main.addChild(this.holder);
	
	this.continueBtn = new PIXI.Sprite.fromFrame("continue.png");
	this.continueBtn.anchor.set(0.5);
	this.continueBtn.x = screenManager.centerX;
	this.continueBtn.y = screenManager.windowH + this.continueBtn.height * 0.5;
	
	layers.main.addChild(this.continueBtn);
	
	this.continueBtn.buttonMode = true;
	this.continueBtn.interactive = true;

	this.continueBtn.on('pointerdown', function() { self.onContinueClick(); });
	
	this.bulletsImg = new PIXI.Sprite.fromFrame("bulletFull.png");
	this.bulletsImg.anchor.set(0.5);
	this.bulletsImg.rotation = Math.PI * 0.5;
	this.bulletsImg.x = screenManager.windowW - 15;
	this.bulletsImg.y = screenManager.windowH - 38;
	
	this.bulletsTxt = new PIXI.BitmapText("6/6", {font: "30px Coolvetica30" });
	this.bulletsTxt.tint = 0xFED844;
	this.bulletsTxt.anchor.set(1, 0);
	this.bulletsTxt.x = screenManager.windowW - 30;
	this.bulletsTxt.y = screenManager.windowH - 40;
	
	this.reloadTxt = new PIXI.Sprite.fromFrame("reload.png");
	this.reloadBar = new PIXI.Sprite.fromFrame("reloadBar.png");
	
    layers.ui.addChild(this.bulletsTxt);
    layers.ui.addChild(this.bulletsImg);
    layers.ui.addChild(this.reloadTxt);
    layers.ui.addChild(this.reloadBar);
}

Gun.prototype.resize = function() {
	this.continueBtn.x = screenManager.centerX;
	this.continueBtn.y = screenManager.windowH - this.continueBtn.height * 0.5;
	
	this.bulletsImg.x = screenManager.windowW - 15;
	this.bulletsImg.y = screenManager.windowH - 38;
	
	this.bulletsTxt.x = screenManager.windowW - 30;
	this.bulletsTxt.y = screenManager.windowH - 40;
	
	this.reloadTxt.x = screenManager.windowW - 80;
	this.reloadTxt.y = screenManager.windowH - 40;
	
	this.reloadBar.x = screenManager.windowW - 76;
	this.reloadBar.y = screenManager.windowH - 40;
}

Gun.prototype.onDragStart = function(e) {
	if(director.state == STATE_SHOOT) {
		this.dragged = true;
		observer.event(EVENT_GUN_DRAG);
		var pos = e.data.getLocalPosition(layers.main);
		input.globalX = pos.x;
		input.globalY = pos.y;
		this.dragStart.x = input.globalX;
		this.dragStart.y = input.globalY;
		e.stopPropagation();
	}
}

Gun.prototype.onDragStop = function() {
	this.dragged = false;
}

Gun.prototype.onContinueClick = function() {
	if(director.currentMode == "assemble") {
		director.nextLevel();
	}else{
		director.goCreative();
	}
}

Gun.prototype.setStats = function() {
	var i, p;	
	this.magSize = 0;
	this.fireMode = "semi";
	this.accuracy = 0;
	this.fireRate = 0;
	this.reloadTime = 0;
	
	for(i = director.partList.length - 1; i >= 0; i--) {
		p = director.partList[i];
		if(p.data.bullets != null) this.magSize += p.data.bullets;
		if(p.data.accuracy != null) this.accuracy += p.data.accuracy;
		if(p.data.reloadTime != null) this.reloadTime += p.data.reloadTime;
		if(p.data.fireRate != null) this.fireRate += p.data.fireRate;
		if(p.data.fireMode != null) {
			if(this.fireMode == "semi") this.fireMode = p.data.fireMode;
			if(p.data.fireMode == "burst" && this.fireMode == "semi") this.fireMode = p.data.fireMode;
			if(p.data.fireMode == "auto" && (this.fireMode == "burst" || this.fireMode == "semi")) this.fireMode = p.data.fireMode;
		}
	}
	
	this.magBullets = this.magSize;
	
	//set pivot to grip position
	var grips = director.findPartType("grip");
	if(grips.length > 0) {
		this.holder.pivot.x = grips[0].holder.x;
		this.holder.pivot.y = grips[0].holder.y;
		this.x = this.holder.x = grips[0].holder.x;
		this.y = this.holder.y = grips[0].holder.y;
	}else{
		this.holder.pivot.x = director.partList[0].holder.x;
		this.holder.pivot.y = director.partList[0].holder.y;
		this.x = this.holder.x = director.partList[0].holder.x;
		this.y = this.holder.y = director.partList[0].holder.y;
	}
	
	//set smoke emitters
	var barrels = director.findPartType("barrel");
	var e;
	this.smokeEmitterList = [];
	for(i = 0; i < barrels.length; i++) {
		this.smokeEmitterList.push(new Emitter(["smoke.png"], layers.bg, 0, 0.1, 0.75, -10, 20, true)); //(imgs, layer, rate, angle, amp, v, life, explode)
	}
}

Gun.prototype.shoot = function() {
	if(this.cooldown == 0) {
		var barrelList = director.findPartType("barrel");
		var i, e;
		
		this.currentBurst++;
		
		if(this.magBullets == 0) {
			this.reload();
			
		}else{
			this.holder.rotation = 0;
			var bulletCaseGenerators = director.findBulletCaseGenerators();
			
			for(i = 0 ; i < barrelList.length; i++) {
				if(this.magBullets > 0) {
					var barrel = barrelList[i];
					if(i == 0) sound.sfx(barrel.data.sfx);
					director.bulletsFired++;
					if(director.bulletsFired == 1000) {
						achievements.earn("bullets1000");
					}
					if(director.prevState == STATE_CREATIVE) {
						achievements.earn("creativeShoot");
					}
					barrel.image.x += 10;
					this.holder.x += 10;
					var px = -this.holder.pivot.x + this.holder.x + barrel.holder.x - barrel.image.width * 0.5 - 10;
					var py = -this.holder.pivot.y + this.holder.y + barrel.holder.y;
					var vx = -40;
					var vy = (1 - barrel.data.accuracy) * (-10 + 20 * Math.random());
					
					if(barrel.data.fireMode == "shotgun") {
						for(var j = 0; j < 5; j ++) {
							vx = -40 + Math.random() * 10;
							vy = -10 + 20 * Math.random();
							this.bulletList.push(new Bullet(px, py, vx, vy, barrel.data.damage));
						}
					}else{
						this.bulletList.push(new Bullet(px, py, -40, vy, barrel.data.damage)); 
					}
					this.magBullets--;
					this.updateBulletCounter();
					this.bulletsImg.scale.x = this.bulletsImg.scale.y = 1.2;
					
					//bullet case
					if(bulletCaseGenerators.length > 0) {
						var bcg = bulletCaseGenerators[0];
						var bcData = bcg.data.bulletCase.split(",");
						var bcpx = -this.holder.pivot.x + this.holder.x + bcg.holder.x - bcg.image.width * 0.5 + parseInt(bcData[0]);
						var bcpy = -this.holder.pivot.y + this.holder.y + bcg.holder.y - bcg.image.height * 0.5 + parseInt(bcData[1]);
						this.bulletCaseList.push(new BulletCase(bcpx, bcpy));
						bulletCaseGenerators.splice(0, 1);
					}
					
					//muzzle flash
					this.muzzleList.push(new Muzzle(px, py));
					
					//smoke emitters
					/*for(var j = 0; j < this.smokeEmitterList.length; j++) {
						e = this.smokeEmitterList[j];
						e.x = px;
						e.y = py;
						e.timer = 30;
					}*/
					
					observer.event(EVENT_GUN_SHOOT);
				}
			}
			this.cooldown = this.fireRate;
		}
	}
}

Gun.prototype.reload = function() {
	this.magBullets = this.magSize;
	this.updateBulletCounter();
	this.cooldown = this.reloadTime;
}

Gun.prototype.isPartLegal = function(part) {
	if(director.partList.length == 0 || (director.partList.length == 1 && director.partList[0] == part)) return true;
	
	var i, p;	
	for(i = director.partList.length - 1; i >= 0; i--) {
		p = director.partList[i];
		if(p.isCollidingWith(part.holder) && p != part) return true;
	}
	return false;
}

Gun.prototype.clear = function() {
	var i, p;
	for(i = director.partList.length - 1; i >= 0; i--) {
		p = director.partList[i];
		p.remove();
	}
	
	this.x = this.y = this.holder.pivot.x = this.holder.pivot.y = this.holder.x = this.holder.y = 0;
}

Gun.prototype.updateBulletCounter = function() {
	this.bulletsTxt.text = this.magBullets + "/" + this.magSize;
}

Gun.prototype.update = function() {
	//UPDATE BULLETS
	var i, b, bc, m, e;
	for(i = this.bulletList.length - 1 ; i >= 0; i--) {
		b = this.bulletList[i];
		b.update();
	}
	
	//UPDATE BULLET CASES
	for(i = this.bulletCaseList.length - 1; i >= 0; i--) {
		bc = this.bulletCaseList[i];
		bc.update();
	}
	
	//UPDATE MUZZLE FLASH
	for(i = this.muzzleList.length - 1; i >= 0; i--) {
		m = this.muzzleList[i];
		m.update();
	}
	
	//UPDATE SMOKE PARTICLES
	for(i = 0; i < this.smokeEmitterList.length; i++) {
		e = this.smokeEmitterList[i];
		e.update();
	}
	
	//UPDATE COLLISION PARTICLES
	this.collisionEmitter.update();
	
	//DRAG GUN
	if(this.dragged && director.state == STATE_SHOOT) {
		var difx = this.dragStart.x - input.globalX;
		var dify = this.dragStart.y - input.globalY;
		
		this.dragStart.x = input.globalX;
		this.dragStart.y = input.globalY;
		
		this.x -= difx;
		this.y -= dify;
	}
	
	
	var vy = this.y - this.holder.y;
	var vx = this.x - this.holder.x;
	this.holder.x += vx * 0.4;
	this.holder.y += vy * 0.4;
	
	this.holder.rotation = vy * 0.005 - vx * 0.005;
	
	if(input.pressed && this.cooldown == 0 && (this.fireMode == "auto" || (this.fireMode == "burst" && this.currentBurst < 3)) && director.state == STATE_SHOOT) this.shoot();
	
	this.cooldown--;
	if(this.cooldown < 0) this.cooldown = 0;
	
	//bullets IMG
	this.bulletsImg.y = input.globalY + 11;
	this.bulletsTxt.y = input.globalY - 2;
	if(this.bulletsTxt.y > screenManager.windowH - 48) {
		this.bulletsImg.y = screenManager.windowH - 46 + 11;
		this.bulletsTxt.y = screenManager.windowH - 48;
	}
	if(this.bulletsTxt.y < 58) {
		this.bulletsImg.y = 60 + 11;
		this.bulletsTxt.y = 58;
	}
	
	this.bulletsImg.scale.x += (1 - this.bulletsImg.scale.x) * 0.3;
	this.bulletsImg.scale.y = this.bulletsImg.scale.x;
	
	this.bulletsImg.visible = this.bulletsTxt.visible = (director.state == STATE_SHOOT);
	if(this.cooldown > 0) {
		this.bulletsTxt.alpha = 0.5;
	}else{
		this.bulletsTxt.alpha = 1;
	}
	
	//reload bar
	this.reloadTxt.visible = (director.state == STATE_SHOOT && (this.cooldown > this.fireRate || this.magBullets == 0));
	this.reloadBar.visible = (this.cooldown > this.fireRate);
	this.reloadBar.scale.x = (this.cooldown / this.reloadTime);
	this.reloadTxt.y = this.bulletsImg.y - 35;
	this.reloadBar.y = this.reloadTxt.y - 6;
	
	//CONTINUE BTN
	var dy = screenManager.windowH + this.continueBtn.height * 0.5;
	if(director.state == STATE_SHOOT && (director.currentTargetsDestroyed >= 3 || director.currentMode == "creative")) dy -= this.continueBtn.height;
	this.continueBtn.y += (dy - this.continueBtn.y) * 0.25;
}