/**
 * ...
 * @author lafunk
 */

var main;
var game;
var partsJson;
var levelsJson;
var tutorialJson;
var debug = false;

var isLoaded = false;
var isLagged = false;
var isMobile = true;
var isPendingInterstitial = false;
var adId = "ca-app-pub-6893876361346206/9431012630";
//ca-app-pub-6893876361346206/9431012630 //android
//ca-app-pub-6893876361346206/6286932409 //ios

var windowW = 800;
var windowH = 600;

//FPS SETTINGS
var fps = 100; //limit framerate
var frameLength = 1000 / fps;
var lastFrame = 0;

window.onerror = function(msg, url, linenumber) {
    alert('Error message: '+msg+'\nURL: '+url+'\nLine Number: '+linenumber);
    return true;
}

//ADMOB
function onDeviceReady() {
	document.removeEventListener('deviceready', onDeviceReady, false);
	admob.setOptions({
        publisherId: "pub-6893876361346206",
        interstitialAdId: adId
	});
	
	document.addEventListener(admob.events.onAdLoaded, onAdLoadedEvent);
	prepareInterstitial();
	
}

function onAdLoadedEvent(e) {
    if (e.adType === admob.AD_TYPE.INTERSTITIAL) {
        isPendingInterstitial = true;
    }
}

function prepareInterstitial() {
	
	if (!isPendingInterstitial) { // We won't ask for another interstitial ad if we already have an available one
        admob.requestInterstitialAd({
            autoShowInterstitial: false
        });
    }
}

function showInterstitial() {
    if (isPendingInterstitial) {
        admob.showInterstitialAd(function () {
			isPendingInterstitial = false;
            prepareInterstitial();
        });
    } else {
       prepareInterstitial();
    }
}

//MAIN
function Main() {
	if(isLagged) LaggedAPI.init('gun_buildertwo_init','lagdevaF3001');
	
	this.bgColor = 0x556270;
	this.loadPercent = 0;
	
	var loader = PIXI.loader; //new PIXI.AssetLoader(["IMG/tiles.json"]);
	loader.add("IMG/Coolvetica30.fnt");
	loader.add("tiles", "IMG/tiles.json");
	loader.add("IMG/overlay.png");
	
	loader.once("complete", function() {	
		$.getJSON( "JSON/tutorial.json", function(d) { //load JSON game data
			tutorialJson = d;
			if(levelsJson != null && partsJson != null) main.loaded();
		});
		
		$.getJSON( "JSON/parts.json", function(d) { //load JSON game data
			partsJson = d;
			if(levelsJson != null && tutorialJson != null) main.loaded();
		});
		
		$.getJSON( "JSON/levels.json", function(d) { //load JSON game data
			levelsJson = d;
			if(partsJson != null && tutorialJson != null) main.loaded();
		});
		
		
	});
	loader.load();
}
	
//PRELOADER
Main.prototype.loaded = function() {
	isLoaded = true;
	
	this.setUpPixi();
	
	game = new Game();
};


Main.prototype.updatePreload = function() {
	if(this.loadPercent < 99) this.loadPercent += 0.5;
	$(".loadBar").css("width", this.loadPercent + "%");
	$(".loadBar").html(Math.floor(this.loadPercent) + "%");
	
	if(partsJson != null && levelsJson != null) {	
		$(".loadBar").css("width", "100%");
		$(".loadBar").html("TAP TO CONTINUE");
	}
};

function loopMain() {
	if(director == null || director.state == "i") {
		main.updatePreload();
		window.requestAnimationFrame(loopMain);
	}
}
	
////////// INIT PIXI ///////////
Main.prototype.setUpPixi = function() {
	//PIXI.scaleModes.DEFAULT = PIXI.scaleModes.NEAREST; //pixel graphics
	this.app = new PIXI.Application(windowW, windowH, {backgroundColor : 0x556270 });
	this.stage = this.app.stage;
	document.body.appendChild(this.app.view);
};


///////////////////STATS////////////////////
if(debug) {
	var stats = new Stats();
	stats.setMode(0); // 0: fps, 1: ms

	// Align top-right
	stats.domElement.style.position = 'absolute';
	stats.domElement.style.left = 'calc(100% - 80px)';
	stats.domElement.style.top = '0px';
}


////////////////DOM READY/////////////
$(document).ready(function() { 
	if(debug) document.body.appendChild( stats.domElement );
	main = new Main(); //Turn on
	loopMain();
	var img = document.createElement('img');
	img.src = "IMG/sponsor.png";
	img.onclick = function(e) {
		e.stopPropagation();
		window.open('http://www.lagged.com','_blank');
	 }
	$("body")[0].appendChild(img);
	
});