function MenuItem(_parent, _data) {
	this.parent = _parent;
	this.data = _data;
	this.dragged = false;
	this.illegal = false;
	
	this.index = _parent.partsList.length;
	
	this.holder = new PIXI.DisplayObjectContainer();
	
	this.bg = new PIXI.Sprite.fromFrame("partBg.png");
	this.bg.anchor.set(0.5);
	
	this.image = new PIXI.Sprite.fromFrame("parts/" + _data.texture + ".png");
	this.image.anchor.set(0.5);
	
	this.image.scale.x = this.image.scale.y = 0.5;
	
	if(_data.fireMode != null && _data.fireMode == "auto") this.fireMode = new PIXI.Sprite.fromFrame("auto.png");
	if(_data.fireMode != null && _data.fireMode == "burst") this.fireMode = new PIXI.Sprite.fromFrame("burst.png");
	
	this.holder.y = this.getPos();
	
	this.holder.addChild(this.bg);
	this.holder.addChild(this.image);
	if(this.fireMode != null) {
		this.fireMode.x = 14;
		this.fireMode.y = 10;
		this.holder.addChild(this.fireMode);
	}
	layers.menu.addChild(this.holder);
	
	this.holder.buttonMode = true;
	this.holder.interactive = true;
	
	var self = this;
	this.holder.on('pointerdown', function() { self.onDragStart(); })
    this.holder.on('pointerup',  function() { self.onDragStop(); });
    this.holder.on('pointerupoutside',  function() { self.onDragStop(); });
}

MenuItem.prototype.onDragStart = function() {
	this.dragged = true;
	director.currentDraggedPart = this;
	
	this.holder.alpha = 0.6;
}

MenuItem.prototype.onDragStop = function() {
	this.dragged = false;
	
	if(this.holder.x > 180 && (gun.isPartLegal(this) || level.readyForCompletion != null)) {
		var p = new Part(this.data, this.holder.x, this.holder.y);
		
		if(level.tryCompletePart(p)) p.holder.scale.x = p.holder.scale.y = 1.25;
		
		this.holder.x = -100;
		this.holder.y = this.getPos();
	}
	
	director.currentDraggedPart = null;
}

MenuItem.prototype.getPos = function() {
	var py = this.index * 64 + 75 + this.parent.header.y;
	//if(menu != null) py -= (menu.height + screenManager.windowH) * menu.scroll.ratio;
	return py;
}

MenuItem.prototype.isActive = function() {
	return (director.currentLevel > this.data.unlock || (director.state == STATE_BUILD && director.currentLevel >= this.data.unlock));
}

MenuItem.prototype.update = function() { 
	if(this.parent.opened && this.isActive() && this.parent.tick > this.index * 2) {
		this.holder.visible = true;
		if(!this.dragged) this.holder.alpha += (1 - this.holder.alpha) * 0.3;
	}else{
		this.holder.x += (0 - this.holder.x) * 0.25;
		this.holder.alpha = 0;
		this.holder.visible = false;
	}
	
	if(this.dragged) { 
		this.holder.x = input.globalX;
		this.holder.y = input.globalY;
		this.bg.alpha = 0;
	}else{
		this.holder.x += (85 - this.holder.x) * 0.25;
		this.holder.y += (this.getPos() - this.holder.y) * 1;
		this.bg.alpha = 1;
	}
	
	if(this.holder.x > 180) {
		this.image.scale.x += (1 - this.image.scale.x) * 0.25;
	}else{
		this.image.scale.x += (0.5 - this.image.scale.x) * 0.25;
	}
	this.image.scale.y = this.image.scale.x;
}