function Part(_data, x, y) {
	this.data = _data;
	this.dragged = false;
	this.illegal = false;
	this.completed = false;
	
	this.holder = new PIXI.DisplayObjectContainer();
	this.image = new PIXI.Sprite.fromFrame("parts/" + _data.texture + ".png");
	this.image.anchor.set(0.5);
	
	this.holder.addChild(this.image);
	this.holder.x = x;
	this.holder.y = y;
	this.holder.z = _data.zOrder;
	
	gun.holder.addChild(this.holder);
	
	gun.holder.children.sort(function(a, b) { if (a.z < b.z) return -1;  if (a.z > b.z) return 1;  return 0; });
	
	this.width = this.holder.width;
	this.height = this.holder.height;
	
	var self = this;
	
	this.holder.buttonMode = true;
	this.holder.interactive = true;
	this.holder.on('pointerdown', function(e) { self.onDragStart(e); });
    this.holder.on('pointerup',  function() { self.onDragStop(); });
    this.holder.on('pointerupoutside',  function() { self.onDragStop(); });
	
	director.partList.push(this);
}

Part.prototype.onDragStart = function(e) {
	if((!this.completed && director.state == STATE_BUILD) || director.state == STATE_CREATIVE) {
		this.dragged = true;
		director.currentDraggedPart = this;
		this.holder.alpha = 0.6;
		e.stopPropagation();
	}
}

Part.prototype.onDragStop = function() {
	this.dragged = false;
	
	this.holder.alpha = 1;
	
	if(level.readyForCompletion == null && (this.holder.x < 180 || !gun.isPartLegal(this))) {
		this.remove();
	}else{
		if(level.tryCompletePart(this)) this.holder.scale.x = this.holder.scale.y = 1.25;
	}
	
	director.currentDraggedPart = null;
}

Part.prototype.isCollidingWith = function(image) {
	var rect1 = {x: this.holder.x - this.width * 0.5, y: this.holder.y - this.height * 0.5, width: this.width, height: this.height}
	var rect2 = {x: image.x - image.width * 0.5, y: image.y - image.height * 0.5, width: image.width, height: image.height}

	if (rect1.x < rect2.x + rect2.width &&
	   rect1.x + rect1.width > rect2.x &&
	   rect1.y < rect2.y + rect2.height &&
	   rect1.height + rect1.y > rect2.y) {
		return true;
	}
	return false;
}

Part.prototype.remove = function() {
	director.partList.splice(director.partList.indexOf(this), 1);
	gun.holder.removeChild(this.holder);
}

Part.prototype.update = function() {
	if(this.dragged) {
		this.holder.x = input.globalX;
		this.holder.y = input.globalY;
	}
	
	this.image.x += (0 - this.image.x) * 0.3;
	
	if(this.holder.x < 180) {
		this.holder.scale.x += (0.5 - this.holder.scale.x) * 0.2;
	}else{
		this.holder.scale.x += (1 - this.holder.scale.x) * 0.25;
	}
	this.holder.scale.y = this.holder.scale.x;
	
	if(this.holder.scale.x > 1.02 || this.image.x > 0.02) {
		var v = (this.holder.scale.x - 1) * 1.5 + this.image.x * 0.05;
		gun.flashFilter.matrix = 	[1, v, v, v, 0, 
									v, 1, v, v, 0,
									v, v, 1, v, 0,
									0, 0, 0, 1, 0];
									
		this.image.filters = [gun.flashFilter];
	}else{
		this.image.filters = [];
	}
	
}