function Submenu(_parent, _type) {
	this.parent = _parent;
	this.type = _type;
	this.index = _parent.submenuList.length;
	this.prevSub = null; 
	this.tick = 0;
	if(this.index > 0) this.prevSub = _parent.submenuList[this.index - 1];
	
	this.opened = false;
	this.partsList = [];
	
	this.header = new PIXI.Sprite.fromFrame(_type + ".png");
	this.header.x = 20 + 60 - Math.round(this.header.width * 0.5);
	layers.menu.addChild(this.header);
	
	this.header.buttonMode = true;
	this.header.interactive = true;
	var self = this;
	this.header.on('pointerdown', function() { self.onClick(); })
	
	this.createParts();
}

Submenu.prototype.onClick = function() {
	this.toggle();
}

Submenu.prototype.toggle = function() {
	(this.opened) ? this.close() : this.open();
}

Submenu.prototype.open = function() {
	this.tick = 0;
	if(menu.selectedSubmenu != null) menu.selectedSubmenu.close();
	this.opened = true;
	menu.selectedSubmenu = this;
	menu.setHeight();
	sound.sfx("open");
	
	observer.event(EVENT_MENU_OPENED);
}

Submenu.prototype.close = function() {
	this.opened = false;
	menu.selectedSubmenu = null;
	menu.setHeight();
}

Submenu.prototype.createParts = function() {
	var list = embed.parts.findAll(["type"],[this.type]);
	var i;
	for(i = 0 ; i < list.length; i++) {
		this.partsList.push(new MenuItem(this, list[i]));
	}
}

Submenu.prototype.findActiveParts = function() {
	var i, p;
	var list = [];
	for(i = 0 ; i < this.partsList.length; i++) {
		p = this.partsList[i];
		if(p.isActive()) list.push(p);
	}
	return list;
}

Submenu.prototype.update = function() {
	this.tick++;
	
	var i, p;
	var dy = menu.header.y + 80;
	if(this.prevSub != null) {
		dy = this.prevSub.header.y + 50;
		if(this.prevSub.opened) {
			dy += this.prevSub.findActiveParts().length * 64;
			dy += 12;
		}
	}
	
	this.header.visible = (this.findActiveParts().length > 0);
	this.header.y += (dy - this.header.y) * 1;
	
	for(i = 0 ; i < this.partsList.length; i++) {
		p = this.partsList[i];
		p.update();
	}
}