function Target(x, y) {
	this.life = 15;
	this.dead = false;
	this.px = x;
	this.py = y;
	this.vr = 0;
	this.tick = 0;
	this.hurtTime = 0;
	
	this.image = new PIXI.Sprite.fromFrame("target.png");
	this.image.anchor.set(0.5);
	this.image.x = -100;
	this.image.y = y;
	
	this.width = this.image.width;
	this.height = this.image.height;
	this.bulletWidth = 21;
	this.bulletHeight = 12;
	
	layers.main.addChildAt(this.image, 0);
}

Target.prototype.checkCollisions = function() {
	var i, b;
	for(i = gun.bulletList.length - 1; i >= 0; i--) {
		b = gun.bulletList[i];
		if(this.isCollidingWith(b.image)) {
			gun.collisionEmitter.x = b.image.x;
			gun.collisionEmitter.y = b.image.y;
			gun.collisionEmitter.timer = 10;
			
			this.vr = (b.image.y - this.image.y) * 0.002;
			this.image.rotation = (b.image.y - this.image.y) * 0.002;
			b.remove();
			this.hurt(b.damage);
		}
	}
}

Target.prototype.isCollidingWith = function(image) {
	var rect1 = {x: this.image.x - this.width * 0.5, y: this.image.y - this.height * 0.5, width: this.width, height: this.height}
	var rect2 = {x: image.x - this.bulletWidth * 0.5, y: image.y - this.bulletHeight * 0.5, width: this.bulletWidth, height: this.bulletHeight}

	if (rect1.x < rect2.x + rect2.width &&
	   rect1.x + rect1.width > rect2.x &&
	   rect1.y < rect2.y + rect2.height &&
	   rect1.height + rect1.y > rect2.y) {
		return true;
	}
	return false;
}

Target.prototype.hurt = function(amount) {
	this.hurtTime = 4;
	this.image.x -= 10;
	this.life -= amount;
	if(this.life <= 0) this.die();
}

Target.prototype.die = function() {
	this.dead = true;
	this.tick = 0;
	director.tick = 0;
	director.currentTargetsDestroyed++;
}

Target.prototype.remove = function() {
	layers.main.removeChild(this.image);
	director.targetList.splice(director.targetList.indexOf(this), 1);
}

Target.prototype.update = function () {
	
	if(this.dead) {
		this.image.tint = 0xFF9999;
		this.image.x += (this.px - 130 - this.image.x) * 0.2;
		this.image.rotation += this.vr;
		this.vr *= 0.95;
		if(this.image.alpha <= 0.05) {
			this.remove();
		}else{
			this.image.alpha += (0 - this.image.alpha) * 0.2;
		}
	}else{
		this.hurtTime--;
		
		if(this.hurtTime > 0) {
			this.image.tint = 0xFF9999;
		}else{
			this.image.tint = 0xFFFFFF;
		}
		this.image.rotation += (0 - this.image.rotation) * 0.2;
		this.image.x += (this.px - this.image.x) * 0.3;
		this.checkCollisions();
	}
}